/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CESubscriptionMgr: a class to handle subscriptions

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CESubscriptionMgr.h,v 1.7.2.2 2007/03/08 13:04:45 adorigo Exp $
*/

#ifndef __GLITE_CE_MONITOR_CLIENT_CESUBSCRIPTIONMGR_H__
#define __GLITE_CE_MONITOR_CLIENT_CESUBSCRIPTIONMGR_H__

#include "glite/ce/monitor-client-api-c/AbsRequest.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include "glite/ce/monitor-client-api-c/Policy.h"
#include <string>
#include <vector>
#include <ctime>
#include <map>

class Subscription {
  
  std::string SubscriptionID;
  time_t ExpirationTime;
  time_t Rate;
  std::string ConsumerURL;
  std::string TopicName;
  std::string endpoint;

 public:
  Subscription(const std::string& a, 
	       const time_t& t, 
	       const time_t& r,
	       const std::string& curl,
	       const std::string& T,
	       const std::string& _endpoint) throw() 
    : SubscriptionID(a), ExpirationTime(t), Rate(r), ConsumerURL(curl), TopicName(T), endpoint(_endpoint) {}

  std::string getSubscriptionID( void ) const throw() { return SubscriptionID; }
  void        setSubscriptionID(const std::string& newid) throw() { SubscriptionID = newid; }
  std::string getConsumerURL( void ) const throw() { return ConsumerURL; }
  time_t      getExpirationTime( void ) const throw() { return ExpirationTime; }
  void        setExpirationTime( const time_t& newT ) throw() { ExpirationTime = newT; }
  std::string getTopicName( void ) const throw() { return TopicName; }
  std::string getEndpoint( void ) const throw() { return endpoint; } 
  time_t      getRate( void ) const throw() { return Rate; }
};

class CESubscriptionMgr : public AbsRequest {

private:

  CESubscriptionMgr(const CESubscriptionMgr&) {}
  CESubscriptionMgr& operator=(const CESubscriptionMgr&) { return *this; }
  std::string consumerURL;
  time_t duration;
  std::vector<Subscription> subids;

public:

  CESubscriptionMgr() throw() : subids() {}
  CESubscriptionMgr(const std::string&, const std::string&) 
    throw(AuthenticationInitException&);

  void list(const std::string&, 
	    std::vector<Subscription>&) throw(GenericException&,
					      TopicNotSupportedException&, 
					      DialectNotSupportedException&,
					      SubscriptionException&,
					      AuthenticationException&,
					      AuthorizationException&,
					      SubscriptionNotFoundException&,
					      ServiceNotFoundException&,
					      GeneralException&);

  std::string update(const std::string&,
	      	     const std::string&,
	      	     const std::string&,
	      	     Topic&,
	      	     Policy&,
	      	     const time_t&
		     ) throw(GenericException&,
			     TopicNotSupportedException&, 
			     DialectNotSupportedException&,
			     SubscriptionException&,
			     AuthenticationException&,
			     AuthorizationException&,
			     SubscriptionNotFoundException&,
			     ServiceNotFoundException&,
			     GeneralException&);
  
  virtual ~CESubscriptionMgr() throw() {}
};

#endif
